package edu.harvard.mgh.purcell.gplink.base_form;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import edu.harvard.mgh.purcell.gplink.mainGUI.*;


/**
 * An extention of JDialog that provides an abstract
 * class to base the forms that construct the command line.
 * @author Kathe Todd-Brown
 *
 */
public abstract class Forms extends JDialog {
	private JTabbedPane fileInputs;
	private BinarySelectPanel binInputs;
	private TextSelectPanel textInputs;
	private AlterPhenoPanel phenoInputs;
	protected JPanel body;
	private OutFilePanel fileOutputs;
	protected ButtonPanel buttons;
	private String filterString;
	private String thresholdString;
	protected JLabel information;
	protected Vector <String> allFiles;
	public MainFrame myFrame;
	
	protected abstract void createBody();
	protected abstract String processBody();
	protected abstract boolean isBodyValid();
	protected abstract void initalize();
	
	/**
	 * Set the threshold back to blank
	 */
	protected void resetThreshold(){
		thresholdString = "";
	}
	
	/**
	 * Set the filter back to blank
	 *
	 */
	protected void resetFilter(){
		filterString = "";
	}
	
	/**
	 * Append the threshold string
	 * @param s String that we want to append to the threshold
	 */
	void addThreshold(String s){
		thresholdString += s;
	}
	
	/**
	 * Append the filter string
	 * @param s String that we want to append to the filter
	 */
	void addFilter(String s){
		filterString += s;
	}
	
	/**
	 * Change the status of the ok button when you change
	 * tabs.
	 */
	private ChangeListener checkTab = new ChangeListener(){
		public void stateChanged(ChangeEvent e) {
			//System.out.println("checkTab@Forms.java: check tab twiggered.");
			isValidCommand();
		}
	};
	
	/**
	 * Check that the form is filled out and have the
	 * ok button reflect this.
	 */
	public void isValidCommand(){
		if(fileInputs == null 
				|| buttons == null 
				|| phenoInputs == null){
			//System.out.println("isValidCommand@Forms.java: stuff is null! returning");
			if(buttons != null)
				buttons.enableOk(false);
			return;
		}
		
		buttons.enableOk(true);
		
		if(phenoInputs.validInput() == false){
			//information.setText("Invalid alternate phenotype.");
			buttons.enableOk(false);
		} else if(fileOutputs.isValidInput() == false){
			//information.setText("Invalid output file.");
			buttons.enableOk(false);
		} else if(fileInputs.getSelectedIndex() == 0 
				&& binInputs.validInput == false){
			//information.setText("Invalid binary input file.");
			buttons.enableOk(false);
		} else if(fileInputs.getSelectedIndex() == 1 
				&& textInputs.validInput == false){
			//information.setText("Invalid standard input file.");
			buttons.enableOk(false);
		} else if(fileInputs.getSelectedIndex() != 0
				&& fileInputs.getSelectedIndex() != 1){
			//information.setText("Invalid input file, \n" +
			//		"selected either binary or text inputs.");
			buttons.enableOk(false);
		}
		information.setText("");
		if(isBodyValid() ==  false){
//			information.setText("Invalid body of the form.");
			isBodyValid();
			buttons.enableOk(false);
		}
	}
	
	
	private void createButtons(){
		buttons = new ButtonPanel(this);
	}
	
	
	private void createInput(){
		fileInputs = new JTabbedPane();
		fileInputs.addChangeListener(checkTab);
		binInputs = new BinarySelectPanel(this);
		textInputs = new TextSelectPanel(this);
		phenoInputs = new AlterPhenoPanel(this);
		fileInputs.addTab("Binary fileset", binInputs);
		fileInputs.addTab("Standard fileset", textInputs);
		fileInputs.addTab("Alternative phenotypes", phenoInputs);
	}
	
	private void createOutput(){
		fileOutputs = new OutFilePanel(myFrame.data, this);
	}
	
	public Forms (MainFrame mf, String title){
		super(mf, title, true);
		myFrame = mf;
		filterString = "";		
		thresholdString = ""; 	//don't overwrite the default thresholds
		information = new JLabel();
		body = new JPanel();
		allFiles = myFrame.data.getFileList();
		initalize();
		createInput();
		createBody();
		createOutput();
		createButtons();
		
		JPanel contents = new JPanel();
		GridBagLayout layout = new GridBagLayout();
		contents.setLayout(layout);
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 1;
		c.weighty = 0.5;
		
		contents.add(new JScrollPane(fileInputs), c);
		c.gridy = 1;
		c.gridx = 0;
		c.weighty = 1;
		
//		add some space so that the words in the body are not
		//...right up against the edge of the form.
		body.setBorder(new EmptyBorder(5,5,5,5));
		contents.add(new JScrollPane(body), c);
		c.gridy = 2;
		c.gridx = 0;
		c.weighty = 0;
		contents.add(fileOutputs, c);
		
		getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS));
		getContentPane().add(contents,c);
		getContentPane().add(information,c);
		getContentPane().add(buttons, c);

		isValidCommand();
		pack();
		setVisible(true);
	}
	
	protected ActionListener cancel = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			dispose();
		}
	};
	
	protected ActionListener filter = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			new FilterDialog(Forms.this);
		}
	};
	
	protected ActionListener threshold = new ActionListener(){
		public void actionPerformed(ActionEvent e){
			new ThresholdDialog(Forms.this);
		}
	};
	
	protected ActionListener process = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			String cline = "";
			
			if(myFrame.data.getPlinkPrefix().trim().length() != 0)
				cline += myFrame.data.getPlinkPrefix() + " ";
			
			cline += processFilename(myFrame.data.getPlinkPath().trim());
			
			if(fileInputs.getSelectedIndex() == 0)
				cline += " " + binInputs.getCommand(false);
			
			if(fileInputs.getSelectedIndex() == 1)
				cline += " " + textInputs.getCommand(false);
			
			cline += " " + phenoInputs.process();
			cline += " " + filterString;
			cline += " " + thresholdString;
			cline += " " + processBody();
			cline += " " + fileOutputs.getCommand();
			
			new PlinkExicute(myFrame, cline);
			dispose();
		}
	};
	
	
	private DocumentListener updateOK_DL = new DocumentListener(){

		public void insertUpdate(DocumentEvent arg0) {
			isValidCommand();
		}

		public void removeUpdate(DocumentEvent arg0) {
			isValidCommand();
		}

		public void changedUpdate(DocumentEvent arg0) {
			isValidCommand();				
		}
		
	};
	
	@SuppressWarnings("serial") 
	protected class myJTextField extends GeneralUtil.UpdatingTextField{
		
		
		public myJTextField(String s, int column){
			super(s, column, updateOK_DL);
		}
		
		public myJTextField(int column){
			super(column, updateOK_DL);
		}
		
		public myJTextField(){
			super(updateOK_DL);
		}
		
		
	}
	
	private ItemListener updateOk = new ItemListener(){
		public void itemStateChanged(ItemEvent arg0) {
			isValidCommand();
			
		}
	};
	
	@SuppressWarnings("serial") 
	protected class myCheckBox extends GeneralUtil.OptionalSelectButton{
		public myCheckBox(String s) {
			super(s, updateOk);
		}

		public myCheckBox(String s, JTextField target){
			super(s, target, updateOk);
		}
		
		public myCheckBox(String s, JTextField target, JButton browser){
			super(s, target, browser, updateOk);
		}
		
		public myCheckBox(String s, JTextField [] target){
			super(s, target, updateOk);
		}
		public myCheckBox(String s, JTextField [] target, JButton [] browser){
			super(s, target, browser, updateOk);
		}
	}
	
	@SuppressWarnings("serial") 
	protected class myRadioButton extends GeneralUtil.RequiredSelection{
		public myRadioButton(String s) {
			super(s, updateOk);
		}

		public myRadioButton(String s, JTextField target){
			super(s, target, updateOk);
		}
		
		public myRadioButton(String s, JTextField target, JButton browser){
			super(s, target, browser, updateOk);
		}
		
		public myRadioButton(String s, JTextField [] target){
			super(s, target, updateOk);
		}
		public myRadioButton(String s, JTextField [] target, JButton [] browser){
			super(s, target, browser, updateOk);
		}
	}
	
	@SuppressWarnings("serial") 
	protected class pickAFileButton extends GeneralUtil.pickAFileButton{
		public pickAFileButton(String suffix, String discription, JTextField reciver){
			super(suffix, discription, reciver, Forms.this);
		}
		
		public pickAFileButton(JTextField reciver){
			super(reciver, Forms.this);
		}
		

	}
	
	/**
	 * add " to the beiging and end of a string
	 * @param fileName a string
	 * @return the given string with " placed at the begining and
	 * end if " does not appear as the first and last charactures
	 */
	public String processFilename(String fileName){
		
		return GeneralUtil.processFilename(fileName);
	}


}
